function [F_feet,Lambda,Lambda_upper,Lambda_lower,Active_Upper,Active_Lower]=Cbot_Analysis(Ang,Internal_Fraction,Cbot_configuration_file)

% Cbot_Analysis
%
% G=Cbot_Analysis(Ang,Cbot_configuration_file)
%
% Performs matrix analysis of a 2-legged climbing robot
% Ang is a vector of angles of inclination at which to solve for (radians)
% Cbot_configuration_file contains listing of geometric parameters:
%    Mass, Mass of robot (kg)
%    X_hip1, X-distance of Hip 1 (front hip) in Front of CG (m)
%    Y_hip1, Y-distance of Hip 1 (front hip) Below CG (m)
%    X_hip2, X-distance of Hip 2 (rear hip) in Back of CG (m)
%    Y_hip2, Y-distance of Hip 2 (rear hip) Below CG (m)
%    H, height of CG from surface (m)
%    Theta_1, inside contact angle of front leg (90=vertical) (radians)
%    Theta_2, inside contact angle of rear leg (90=vertical) (radians)
%    Mu_1, Coefficient of Friction of front contact
%    Mu_2, Coefficient of Friction of rear contact
%    F_t, Magnitude of External Thrust Vector (N)
%    X_t, X-Distance of External Thrust Vector in Front of CG (m)
%    Y_t, Y-Distance of External Thrust Vector Above CG (m)
%    Theta_t, Angle of External Thrust Vector (wrt Cbot) (90=verical,
%    pointing down) (radians)

% Load geometric configuration
eval(Cbot_configuration_file);

% Define constants
g=9.8;

% Calculate translation and rotation vectors for foot contact points
R_1=[X_hip1+((H-Y_hip1)/tan(Theta_1)) W/2 -H];
RPY_1=[0 -Phi_1 0];
R_2=[-X_hip2-((H-Y_hip2)/tan(Theta_2)) -W/2 -H];
RPY_2=[0 -Phi_2 0];

% Calculate corresponding homogeneous transformation matrices and Jacobians
T_1=transl(R_1)*rpy2tr(RPY_1);
T_2=transl(R_2)*rpy2tr(RPY_2);

J_1=tr2jac(T_1);
J_2=tr2jac(T_2);
J_1_t=J_1';
J_2_t=J_2';

% Ignore Moment Contributions from Jacobians (feet are point contacts),
% compute Overall Jacobian
J=[J_1_t(:,1:3) zeros(6,3) ; zeros(6,3) J_2_t(:,1:3)];

% Calculate W (Equilibrium Equations)
W=[1 0 0 0 0 0 1 0 0 0 0 0 ; ...
   0 1 0 0 0 0 0 1 0 0 0 0 ; ...
   0 0 1 0 0 0 0 0 1 0 0 0 ; ...
   0 0 0 1 0 0 0 0 0 1 0 0 ; ...
   0 0 0 0 1 0 0 0 0 0 1 0 ; ...
   0 0 0 0 0 1 0 0 0 0 0 1 ];

% Calculate new W = W*J, and strike one of the rows (dependent)
W=W*J;
W=W([1:3,5:6],:);

% Calculate null space of W 
N=null(W);

% Make sure Null space is such that internal body force is
% positive-compressive
if N(1)<0
    N=-N;
end

% Build friction constraint matrix
A=[-1 -1 Mu_1  0  0   0  ; ...
   -1  1 Mu_1  0  0   0  ; ...
    1 -1 Mu_1  0  0   0  ; ...
    1  1 Mu_1  0  0   0  ; ...
    0  0   0  -1 -1 Mu_2 ; ...
    0  0   0  -1  1 Mu_2 ; ...
    0  0   0   1 -1 Mu_2 ; ...
    0  0   0   1  1 Mu_2 ; ...
             -N'         ; ...
              N'         ];

for i=1:length(Ang)
    
    % Calculate translation and rotation vectors for gravity vector and
    % external thrust vector
    R_g=[0 0 0];
    RPY_g=[0 Ang(i) 0];
    R_t=[X_t 0 Y_t];
    RPY_t=[0 Theta_t 0];
    
    % Calculate corresponding homogeneous transformation matrices and Jacobians
    T_g=transl(R_g)*rpy2tr(RPY_g);
    T_t=transl(R_t)*rpy2tr(RPY_t);
    
    J_g=tr2jac(T_g);
    J_t=tr2jac(T_t);
    
    % Calculate Externally Applied Force Vector
    F_gravity=[0;0;-Mass*g;0;0;0];
    F_gravity=J_g'*F_gravity;
    F_thrust=[F_t;0;0;0;0;0];
    F_thrust=J_t'*F_thrust;
    
    F=F_gravity+F_thrust;
    %F=[Mass*g*cos(Ang(i));0;Mass*g*sin(Ang(i));0;0];
    F=F([1:3,5:6]);

    % Calculate Left-Hand-Side Lambda Coefficients
    C=A*N;
    c1=C(1,:);
    c2=C(2,:);
    c3=C(3,:);
    c4=C(4,:);
    c5=C(5,:);
    c6=C(6,:);
    c7=C(7,:);
    c8=C(8,:);
    c9=C(9,:);
    c10=C(10,:);

    % AC>=P, what is P?
    P=[0 0 0 0 0 0 0 0 -100 -100]';
    
    % Calculate Boundaries
    Q=(A*pinv(W)*F)+P;
    L1=Q(1)/c1;
    L2=Q(2)/c2;
    L3=Q(3)/c3;
    L4=Q(4)/c4;
    L5=Q(5)/c5;
    L6=Q(6)/c6;
    L7=Q(7)/c7;
    L8=Q(8)/c8;
    L9=Q(9)/c9;
    L10=Q(10)/c10;
    
    % Find which constraints are "greater than" constraints and which are
    % "less than" constraints
    j=1;
    k=1;
    if c1<0 Upper(j)=L1; j=j+1; Constraint(i,1)=1; else Lower(k)=L1; k=k+1; Constraint(i,1)=-1; end
    if c2<0 Upper(j)=L2; j=j+1; Constraint(i,2)=1; else Lower(k)=L2; k=k+1; Constraint(i,2)=-1; end
    if c3<0 Upper(j)=L3; j=j+1; Constraint(i,3)=1; else Lower(k)=L3; k=k+1; Constraint(i,3)=-1; end
    if c4<0 Upper(j)=L4; j=j+1; Constraint(i,4)=1; else Lower(k)=L4; k=k+1; Constraint(i,4)=-1; end
    if c5<0 Upper(j)=L5; j=j+1; Constraint(i,5)=1; else Lower(k)=L5; k=k+1; Constraint(i,5)=-1; end
    if c6<0 Upper(j)=L6; j=j+1; Constraint(i,6)=1; else Lower(k)=L6; k=k+1; Constraint(i,6)=-1; end
    if c7<0 Upper(j)=L7; j=j+1; Constraint(i,7)=1; else Lower(k)=L7; k=k+1; Constraint(i,7)=-1; end
    if c8<0 Upper(j)=L8; j=j+1; Constraint(i,8)=1; else Lower(k)=L8; k=k+1; Constraint(i,8)=-1; end
    if c9<0 Upper(j)=L9; j=j+1; Constraint(i,9)=1; else Lower(k)=L9; k=k+1; Constraint(i,9)=-1; end
    if c10<0 Upper(j)=L10; j=j+1; Constraint(i,10)=1; else Lower(k)=L10; k=k+1; Constraint(i,10)=-1; end
    
    %temp(i,:)=[L1 L2 L3 L4 L5 L6 L7 L8 L9 L10];
    
    % Find active constraints
    j=1;
    k=1;
    Min=Inf;
    Max=-Inf;
    for z=1:10
        if Constraint(i,z)==1
            if Upper(j)<Min
                Min=Upper(j);
                Active_Upper(i)=z;
            end
            j=j+1;
        end
        if Constraint(i,z)==-1
            if Lower(k)>Max
                Max=Lower(k);
                Active_Lower(i)=z;
            end
            k=k+1;
        end
    end
    
    % Find feasible range for Lambda
    L_min(i)=max(Lower);
    L_max(i)=min(Upper);
    
    % Find best lambda within range and solve
    if (L_min(i)<L_max(i))
        Slip(i)=0;
        Lambda(i)=L_min(i)+((L_max(i)-L_min(i))*Internal_Fraction);
        Lambda_upper(i)=L_max(i);
        Lambda_lower(i)=L_min(i);
        F_feet(:,i)=inv([W;N'])*[-F;Lambda(i)];
    else
        Slip(i)=1;
        Lambda(i)=NaN;
        Lambda_upper(i)=NaN;
        Lambda_lower(i)=NaN;
        F_feet(:,i)=[NaN;NaN;NaN;NaN;NaN;NaN];
    end
end

%save workspace;
F_feet=F_feet';
